package edu.upvehu.gbg.babel;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;

/**
 * Clase que permite que un objeto contenga el modelo de un lenguajes
 * conforme a las especificaciones dadas en la práctica 5 de TAP
 * @author Germán
 */
public class Language {
    /**
     * El modelo del lenguaje es un histograma de frecuencias de caracteres
     */
    private final Map<Character,Double> frecuencias=new TreeMap<>();

    /**
     * Código identificador del lenguaje
     */
    private final String id;

    /**
     * Descriptor del lenguaje
     */
    private final String descriptor;
    
    /**
     * 
     * @param id
     * @param descriptor
     * @param filename 
     * @throws java.io.IOException 
     */
    public Language(String id, String descriptor, String filename) throws IOException {
        //Consideramos el recurso (fichero) que se nos indica...
        try (BufferedReader br= new BufferedReader(new FileReader(filename))){  
            this.id=id;
            this.descriptor=descriptor;
            int n=0; //...contaremos cuantos caracteres leemos...
            //...y lo hacemos caracter a caracter llevando la cuenta en el mapa...
            for (int c; (c=br.read())!=-1; n++ ){
                Double d=frecuencias.get((char)c);
                frecuencias.put((char)c, d==null?1.0:d+1);} 
            //...y terminamos normalizandolo dividiendo cada cuanta por la total (n)
            for (Character ch:frecuencias.keySet()) frecuencias.put(ch,frecuencias.get(ch)/n);}
    }

    /**
     * Getter del código identificador del lenguaje 
     * @return el identificador del lenguaje
     */
    public String getId() {return id;}
    
    /**
     * Getter del descriptor del lenguaje
     * @return el descriptor del lenguaje
     */
    public String getDescriptor(){return descriptor;}
    
    /**
     * Medida de la similaridad entre dos lenguajes
     * @param a lenguaje a comparar con <code>b</code>
     * @param b lenguaje a comparar con <code>a</code>
     * @return el valor de la similaridad
     */
    public static double similarity(Language a, Language b){
        //Recorriendo los caracteres de un lenguaje, si están en el otro, se acumula el mínimo de sus frecuencias
        double similarity=0.0;
        for (Character ch:a.frecuencias.keySet())  
            if (b.frecuencias.get(ch)!=null)
                similarity+=Math.min(a.frecuencias.get(ch),b.frecuencias.get(ch));
        return similarity;
    }

    /**
     * Representación textual de un objeto Language
     * @return 
     */
    @Override
    public String toString() {return "Language{" + "id=" + id + ", descriptor=" + descriptor + '}';}
    
}
